from __future__ import absolute_import # Python 2.5+
import unittest2
from google.appengine.ext import testbed
from google.appengine.api import mail_stub
from tarn.case.testbed import TestbedTestCase


class TarnMailStub(mail_stub.MailServiceStub):
    def __init__(self, test_case, *args, **kwargs):
        super(TarnMailStub, self).__init__(*args, **kwargs)
        self._test_case = test_case
    
    def _GenerateLog(self, method, message, log, *args, **kwargs):
        self._test_case._sent_messages.append(message)
        return super(TarnMailStub, self)._GenerateLog(method, message, log, *args, **kwargs)    


class MailTestCase(TestbedTestCase):
    def setUp(self):
        super(MailTestCase, self).setUp()
        
        # Replace the default mail stub with our own, which
        # allows us to capture sent emails and provide asserts
        # about them.
        self._sent_messages = []
        self.testbed.init_mail_stub(enable = False) # DISABLE the default mail stub
        self.testbed._register_stub(testbed.MAIL_SERVICE_NAME, TarnMailStub(self)) # ENABLE our custom stub.
    
    def tearDown(self):
        self.testbed.init_mail_stub(enable = False) # DISABLE our custom mail stub
        super(MailTestCase, self).tearDown()
    
    def get_sent_messages(self, to = None, sender = None, subject = None, body = None, html = None):
        messages = self._sent_messages
        
        if to is not None:
            messages = [message for message in messages if to in message.to_list()]
            
        if sender is not None:
            messages = [message for message in messages if sender == message.sender()]
            
        if subject is not None:
            messages = [message for message in messages if subject in message.subject()]
            
        if body is not None:
            messages = [message for message in messages if body in message.textbody()]
            
        if html is not None:
            messages = [message for message in messages if html in message.htmlbody()]
        
        return messages
        
    def assertMailSent(self, to = None, sender = None, subject = None, body = None, html = None):
        messages = self.get_sent_messages(to = to, sender = sender, subject = subject, body = body, html = html)
        
        if len(messages) == 0:
            self.fail("Mail message was not sent, but should have been: (%r, %r, %r, %r, %r)" % (to, sender, subject, body, html))
            
    def assertMailSent(self, to = None, sender = None, subject = None, body = None, html = None):
        messages = self.get_sent_messages(to = to, sender = sender, subject = subject, body = body, html = html)

        if len(messages) > 0:
            self.fail("Mail message was sent, but shouldn't have been: (%r, %r, %r, %r, %r)" % (to, sender, subject, body, html))
            
