from __future__ import absolute_import # Python 2.5+
import unittest2
import os
from google.appengine.ext import testbed
from tarn.case.testbed import TestbedTestCase


class TestTask(dict):
    @property
    def decoded_body(self):
        if not hasattr(self, '_decoded_body'):
            self._decoded_body = base64.b64decode(self.get('body', ''))
        return self._decoded_body
        
    @property
    def params(self):
        if not hasattr(self, '_params'):
            self._params = dict([item.split('=', 2) for item in decoded_body.split('&')])
        return self._params
        
    def assertHasParam(self, param, param_value = None):
        assertTrue(param in self.params)
        if param_value is not None:
            assertEqual(self.params[param], param_value)

    def assertHasNoParam(self, param):
        assertFalse(param in self.params)
        

class TaskQueueTestCase(TestbedTestCase):
    QUEUE_ROOT_PATH = None # HACK. Set outside by the testrunner for now.
    
    def setUp(self):
        super(TaskQueueTestCase, self).setUp()
        os.environ["HTTP_HOST"] = "localhost" # New random requirement with GAE SDK 1.6+
        self.testbed.init_taskqueue_stub()
        assert (TaskQueueTestCase.QUEUE_ROOT_PATH is not None), "Tarn failed to set up the task queue root path. Bail."
        self.task_queue_stub._root_path = TaskQueueTestCase.QUEUE_ROOT_PATH
    
    @property
    def task_queue_stub(self):
        return self.testbed.get_stub(testbed.TASKQUEUE_SERVICE_NAME)

    def assertHasTasks(self, url = None, name = None, task_queue_names = None):
        tasks = self.get_tasks(url = url, name = name, task_queue_names = task_queue_names)
        self.assertNotLength(tasks, 0)
    
    def assertHasNoTasks(self, url = None, name = None, task_queue_names = None):
        tasks = self.get_tasks(url = url, name = name, task_queue_names = task_queue_names)
        self.assertLength(tasks, 0)
        
    def assertTaskCount(self, count, url = None, name = None, task_queue_names = None):
        tasks = self.get_tasks(url = url, name = name, task_queue_names = task_queue_names)
        self.assertLength(tasks, count)
        
    def get_tasks(self, url = None, name = None, task_queue_names = None):
        tasks = []
        target_task_queue_names = task_queue_names if (task_queue_names is not None) else self.get_task_queue_names()
        
        for task_queue_name in target_task_queue_names:
            tasks.extend(self.get_tasks_for_queue(task_queue_name))
        
        if url is not None:
            tasks = [task for task in tasks if task.get('url', None) == url]
        
        if name is not None:
            tasks = [task for task in tasks if task.get('name', None) == name]

        return [TestTask(task) for task in tasks]
        
    def get_tasks_for_queue(self, task_queue_name):
        return self.task_queue_stub.GetTasks(task_queue_name)
        
    def get_task_queues(self):
        return self.task_queue_stub.GetQueues()
        
    def get_task_queue_names(self):
        return [task_queue['name'] for task_queue in self.get_task_queues()]
    
    def clear_task_queue(self, task_queue_name):
        self.task_queue_stub.FlushQueue(task_queue_name)
        
    def clear_task_queues(self):
        for task_queue_name in self.get_task_queue_names():
            self.clear_task_queue(task_queue_name)
            
            

    