#
# TARN: Test Appengine Right Now!
#
# It was dark and stormy night, lit only by the glow of a MacBook LCD. The hacker 
# loomed at the edge of the abyss, his code a masterful symphony but his test 
# tools a cruel and spiteful joke. 
#
# TARN is a plugin for nose to properly test app engine apps. It is designed
# to work happily with the new testbed features supplied in SDK 1.4.3+, and 
# to work with the included 'wiseguy' WSGI app runner for integration testing.
#
# In some sense, this part is a re-implementation of nosegae. You'll certainly see
# a lot of carryover, like options and corner-case bugfixes. But where nosegae
# outright failed to actually work in countless scenarios, TAWNY WILL PREVAIL.
#
# So: many props to the nosegae crew -- may your work live on!
#

from __future__ import absolute_import # Python 2.5+
import os
import os.path
import sys
import tempfile
import nose
import nose.plugins
import logging
import warnings


class TarnPlugin(nose.plugins.Plugin):
    """When active, tests are run in App Engine's development environment."""
    name = "tarn"
    
    def options(self, parser, env):
        """Called by nose to allow us to add plugin-specific command-line options."""
        super(TarnPlugin, self).options(parser, env)
        parser.add_option(
            '--gae-root-path', 
            default='/usr/local/google_appengine',
            dest='gae_root_path',
            help='Set the path to the root directory of the Google Application Engine installation',
        )
        parser.add_option(
            '--gae-app-path', 
            default = None, 
            action = 'store', 
            dest = 'gae_app_path',
            help = 'Set the path to the GAE application under test. Default is the working directory',
        )
         
    def configure(self, options, conf):
        """Called by nose to allow us to handle the parsed options."""
        super(TarnPlugin, self).configure(options, conf)
        
        # Bail out if we're not wanted.
        if not self.enabled: 
            return
            
        # Bail out if the version of python is too old
        if sys.version_info[0:2] < (2, 5):
            raise EnvironmentError("Python version must be 2.5 or greater for testing with tarn. You're using %r." % sys.version)

        # Set up key paths
        self.gae_root_path = options.gae_root_path
        self.gae_app_path = options.gae_app_path if (options.gae_app_path is not None) else conf.workingDir
        
        # Bail out if any of our paths are no good...
        if not os.path.exists(os.path.join(self.gae_root_path, 'dev_appserver.py')):
            raise EnvironmentError("Tarn looked for the App Engine SDK in %s, but couldn't find it. Consider using the --gae-root-path command-line option." % self.gae_root_path)
        if not os.path.exists(os.path.join(self.gae_app_path, 'app.yaml')):
            raise EnvironmentError("Tarn looked for your App Engine app in %s, but couldn't find it. Consider using the --gae-app-path command-line option." % self.gae_app_path)
                
        # Ignore sucky warnings from the local GAE SDK
        warnings.filterwarnings('ignore', category = DeprecationWarning)
        
        # Resolve through the symlink for sanity's sake...
        self.gae_real_path = os.path.realpath(self.gae_root_path)
        
    def begin(self):
        """Called by nose before any tests are collected and run."""
        # Add the app engine libraries to our path
        sys.path.insert(0, self.gae_real_path)
        
        # Add the app path itself to our path
        sys.path.append(self.gae_app_path)
        
        # Invoke some internal SDK code to add App Engine stuff to the path 
        # There are two "dev_appserver"s to import from the SDK; this one is
        # the script you invoke from the command line...
        from dev_appserver import fix_sys_path
        fix_sys_path()
        
        # If the user has provided tarn hooks for configuration, load them here
        try:
            from tarn_hooks import configure_for_testing
            configure_for_testing()
        except:
            pass
        from tarn.case.taskqueue import TaskQueueTestCase
        TaskQueueTestCase.QUEUE_ROOT_PATH = self.gae_app_path
        
        # Swallow up App Engine's agressive logging! (See nosegae issue 25)
        rootLogger = logging.getLogger()
        for handler in rootLogger.handlers:
            if isinstance(handler, logging.StreamHandler):
                rootLogger.removeHandler(handler)
        

