#
# TARN: Test Appengine Right Now!
#
# It was dark and stormy night, lit only by the glow of a MacBook LCD. The hacker 
# loomed at the edge of the abyss, his code a masterful symphony but his test 
# tools a cruel and spiteful joke. 
#

from __future__ import absolute_import # Python 2.5+
import os
import os.path
import sys
import unittest2
import webtest
from optparse import OptionParser


class TarnTestRunner(object):
    def run(self, test_path, gae_app_path, gae_sdk_path, verbosity = 1):
        """Call this if you want to manually control the testrunner."""
        # Resolve through symlinks, if any.
        gae_sdk_path = os.path.realpath(gae_sdk_path)
        
        # Validate the presence of the SDK
        if not os.path.exists(os.path.join(gae_sdk_path, "dev_appserver.py")):
            raise EnvironmentError("Tarn could not find the App Engine SDK at %s. Consider using the --gae-sdk-path command-line option." % gae_sdk_path)

        # Set up paths for the App Engine SDK
        sys.path.insert(0, gae_sdk_path)
        import dev_appserver
        dev_appserver.fix_sys_path()

        # Validate our app path (it must have an app.yaml file!)
        if not os.path.exists(os.path.join(gae_app_path, "app.yaml")):
            raise EnvironmentError("Tarn could not find your App Engine application: the directory %s did not contain an app.yaml file. Consider using the --gae-app-path command-line option." % gae_app_path)
            
        # Set up paths for the app itself
        sys.path.insert(0, gae_app_path)
        
        # And for our test root:
        # CONSIDER: do we actually want this?
        if test_path != gae_app_path:
            sys.path.insert(0, test_path)
        
        # If the user has provided tarn hooks for configuration, load them here
        try:
            from tarn_hooks import configure_for_testing
            configure_for_testing()
        except:
            pass
            
        from tarn.case.taskqueue import TaskQueueTestCase
        TaskQueueTestCase.QUEUE_ROOT_PATH = gae_app_path
            
        # Run the tests!
        suite = unittest2.loader.TestLoader().discover(test_path)
        unittest2.TextTestRunner(verbosity=verbosity).run(suite)
    
    def main(self):
        """Called by our tarn command-line script."""
        # Set up option parser
        parser = OptionParser()
        parser.add_option(
            '-s', 
            '--gae-sdk-path', 
            dest = "gae_sdk_path", 
            help = "Set the path to the App Engine SDK, if it is nonstandard.", 
            default = "/usr/local/google_appengine",
        )
        parser.add_option(
            '-a',
            '--gae-app-path',
            dest = "gae_app_path",
            help = "Set the path to your App Engine app. Directory must contain an app.yaml file.",
            default = None,
        )
        parser.add_option(
            '-v',
            '--verbose',
            dest = "is_verbose",
            action = "store_true",
            default = False,
            help = "Get verbose output",
        )
        
        # Parse options
        (options, args) = parser.parse_args()
        if len(args) > 1:
            parser.usage()
            sys.exit(-1)

        # Deal with verbosity verbosely!
        verbosity = 2 if options.is_verbose else 1
        
        # Set up paths
        test_path = os.path.abspath(args[0]) if (len(args) == 1) else os.path.abspath(os.getcwd())
        gae_app_path = options.gae_app_path if (options.gae_app_path is not None) else os.getcwd()
        gae_sdk_path = options.gae_sdk_path
        
        # Run our tests!
        self.run(test_path = test_path, gae_app_path = gae_app_path, gae_sdk_path = gae_sdk_path, verbosity = verbosity)
        
        
        